/*
 * Programming in Java, Jacobs University, 2015
 *
 * Create a frame (a top-level window) and add a button to it.  A
 * frame is not visible until it is set to be visible. Do something
 * when the button is pressed and exit the application if the frame is
 * closed by the window manager.
 *
 * This version uses lambda functions. Unfortunately, not all listener
 * interfaces are functional interfaces so one has to resort to a
 * trick by deriving a suitable functional interface...
 */ 

import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowListener;
import javax.swing.JFrame;
import javax.swing.JButton;

interface WindowClosingListener extends WindowListener
{
    public default void windowActivated(WindowEvent e) {}
    public default void windowClosed(WindowEvent e) {}
    public default void windowDeactivated(WindowEvent e) {}
    public default void windowDeiconified(WindowEvent e) {}
    public default void windowIconified(WindowEvent e) {}
    public default void windowOpened(WindowEvent e) {}
}

public class PressMeLambda
{
    private JFrame frame;
    private JButton button;

    public PressMeLambda()
    {
	frame = new JFrame();
	button = new JButton("Press Me!");
	button.addActionListener(e -> action(e));
	frame.add(button);
	frame.pack();
	frame.addWindowListener((WindowClosingListener)(e) -> System.exit(0));
	frame.setVisible(true);
    }
    
    private void action(ActionEvent e)
    {
	System.out.println("");
	System.out.println("oouch!");
	System.out.println(e);
    }

    public static void main(String[] args)
    {
	PressMeLambda pm = new PressMeLambda();
    }
}
